@@ -7,7 +7,7 @@
 #' @param Cs: soft-margin tuning parameter of the SVM. Defaults to \code{10^c(-3:3)}.
 #' @param Gsub: an adjacency matrix that represents the underlying biological network.
 #' @param pt.pvalue: cut off p value of permutation test
-#' @param pt.step:  permutation test steps
+#' @param aa:  permutation test steps
 #' @param op: optimal on top op% ranked genes
 #' @param a: constant value of random walk kernel
 #' @param p: random walk step(s) of random walk kernel
@@ -27,11 +27,11 @@
 #'
 #' library(netClass)
 #' 
-#' r.stsvm <- stsvm.cv(x=x,x.mi=NULL,y=y, folds=5,Gsub=ad.matrix,  repeats=3, parallel = TRUE, cores=4,DEBUG=TRUE,pt.pvalue=0.05,op=0.9,pt.step=1000,a=1,p=2,allF=TRUE,Cs=10^(-4:4))
+#' r.stsvm <- cv.stsvm(x=x,x.mi=NULL,y=y, folds=5,Gsub=ad.matrix,  repeats=3, parallel = TRUE, cores=4,DEBUG=TRUE,pt.pvalue=0.05,op=0.9,aa=1000,a=1,p=2,allF=TRUE,Cs=10^(-4:4))
 #' mean(r.stsvm$auc)
 
 
-stsvm.cv <- function(x=x, x.mi=NULL,y=y, folds=5,Gsub=matrix(1,100,100),repeats=3, parallel = FALSE, cores=2,DEBUG=TRUE, pt.pvalue=0.05,op=0.85,pt.step=1000,a=1,p=2,allF=TRUE,seed=1234,Cs=10^c(-3:3))
+cv.stsvm <- function(x=x, x.mi=NULL,y=y, folds=5,Gsub=matrix(1,100,100),op.method=c("pt","spb"), repeats=3, parallel = FALSE, cores=2,DEBUG=TRUE, pt.pvalue=0.05,op=0.85, aa=1000,a=1,p=2,allF=TRUE,seed=1234,Cs=10^c(-3:3))
 {
 	multicore <- ("package:parallel" %in% search())
   
@@ -54,10 +54,14 @@ stsvm.cv <- function(x=x, x.mi=NULL,y=y, folds=5,Gsub=matrix(1,100,100),repeats=
 
 	if (folds < 2) stop("folds =< 2.\n")
 	if (folds > n) stop("folds > the number of observations.\n")
+	if(is.null(x.mi))
+	{
+		x= cbind(x,x.mi)
+	}
 	
   	dk.tf = pt.pvalue
   	pred.type="class"
-  	aa=pt.step
+  	#aa=pt.step
   	mapping=NULL
 	ex.sum=x
   	
@@ -112,8 +116,8 @@ stsvm.cv <- function(x=x, x.mi=NULL,y=y, folds=5,Gsub=matrix(1,100,100),repeats=
     
 		if(DEBUG) cat('Starting classification of repeat:',r,'\n')
     
-		if(parallel)	repeat.models <- mclapply(1:folds, classify.stsvm, cuts=cuts, ex.sum=ex.sum, x=x, y=y,cv.repeat=r, DEBUG=DEBUG, Gsub=Gsub,op=op,aa=aa,dk=dk,dk.tf=dk.tf,p=p,a=a,seed=seed,Cs=Cs)
-		else		repeat.models <-   lapply(1:folds, classify.stsvm, cuts=cuts, ex.sum=ex.sum,x=x, y=y,cv.repeat=r, DEBUG=DEBUG, Gsub=Gsub,op=op,aa=aa,dk=dk, dk.tf=dk.tf,p=p,a=a,seed=seed,Cs=Cs)
+		if(parallel)	repeat.models <- mclapply(1:folds, classify.stsvm, cuts=cuts, ex.sum=ex.sum, x=x, y=y,cv.repeat=r, DEBUG=DEBUG, Gsub=Gsub,op.method=op.method,op=op,aa=aa,dk=dk,dk.tf=dk.tf,p=p,a=a,seed=seed,Cs=Cs)
+		else		repeat.models <-   lapply(1:folds, classify.stsvm, cuts=cuts, ex.sum=ex.sum,x=x, y=y,cv.repeat=r, DEBUG=DEBUG, Gsub=Gsub,op.method=op.method,op=op,aa=aa,dk=dk, dk.tf=dk.tf,p=p,a=a,seed=seed,Cs=Cs)
 	
 		#close(pb)
     
@@ -162,7 +166,7 @@ stsvm.cv <- function(x=x, x.mi=NULL,y=y, folds=5,Gsub=matrix(1,100,100),repeats=
 # Return a list with the results of the training model and predcit AUC   
 
 
-classify.stsvm <- function(fold, cuts,ex.sum, x, p,a, y, cv.repeat, DEBUG=DEBUG,Gsub=Gsub, op=op,aa=aa,dk=dk,dk.tf=dk.tf,seed=seed,Cs=Cs)
+classify.stsvm <- function(fold, cuts,ex.sum, x, p,a, y, cv.repeat, DEBUG=DEBUG,Gsub=Gsub,op.method=op.method, op=op,aa=aa,dk=dk,dk.tf=dk.tf,seed=seed,Cs=Cs)
 {
 	gc() 		
 	
@@ -175,39 +179,42 @@ classify.stsvm <- function(fold, cuts,ex.sum, x, p,a, y, cv.repeat, DEBUG=DEBUG,
 	#print(tst)
 	label 	<- sign(as.numeric(y[tst]) - 1.5) # because y is a factor			
 	
-	## train and test the model						
+	train	<- train.stsvm(x=x[trn,], y=y[trn], DEBUG=DEBUG,Gsub=Gsub, op.method=op.method, op=op,aa=aa,dk=dk, dk.tf=dk.tf,seed = seed,Cs=Cs)	    	  	  
+	test	<- predictStsvm(train=train, x=x[tst,], y=y[tst],DEBUG=DEBUG)
+	
+	auc		<- test$auc
+	feat	<- train$feat						
+	
+	if(DEBUG) cat("=> the best AUC   is ", auc, "\t Best features length:  ", length(feat),"\n")					
+	if(DEBUG) cat('Finished fold:',fold,'\n\n')
+
+	gc()	
+	res=list(fold=fold, model=train, auc=auc,feat= feat)
 		
-		
-		xmmiTrn 	<- x[trn,]#,x.mi[trn,])#sig.mi])
-		xmmiTst 	<- x[tst,]#,x.mi[tst,]#sig.mi])
-		
-		sca=TRUE		
-		rank.method="dk"
-		
-		# ranked by smoothed t-statistics  				
-		ranks = getGraphRank(x=x[trn,], y=y[trn], Gsub=dk,sca=sca)	
-		
-						
+	return(res)
+}
+
+
+train.stsvm <- function(x=x, y=y, DEBUG=FALSE,Gsub=Gsub, op.method="sp", op=10,aa=100,dk=dk, dk.tf=0.05,seed = 1234,Cs=10^(-3:3), EN2SY=NULL)
+{
+	# ranked by smoothed t-statistics  	
+	sca=TRUE			
+	ranks = getGraphRank(x=x, y=y, Gsub=dk,sca=sca)							
+	
+	
+	if(op.method=="pt")
+	{
 		topGenes = names(ranks[which(ranks > quantile(ranks,op))])				
-		#ranksI=sort(ranks,decreasing=T)
-		#topGenes = names(ranksI[1:op])
-		#cat("==================\nrange(ranks[topGenes]):",range(ranks[topGenes]),"\n")
-		#topRanks =  getGraphRank1(x=x[trn,topGenes], y=y[trn], Gsub=dk,sca=sca)
 		topRanks =ranks[topGenes]
+		#if(DEBUG) cat("range of topRanks:",range(topRanks),"\n=======================\n")
 		
-		#topRan
-		#topRanks = rank(topRanks)
-		#topRanks=ranks
-		if(DEBUG) cat("range of topRanks:",range(topRanks),"\n=======================\n")
-		
-		ytrn = y[trn]
-		xtrn =xmmiTrn[,topGenes]			
+		ytrn = y
+		xtrn =x[,topGenes]			
 
 		R = aa
 		set.seed(seed)
 		p_ranks=matrix(0,ncol=length(topRanks))[1,]
 		names(p_ranks)=topGenes
-		#cat("length(topRanks) ",length(topRanks),"\n")
 		
 		if(DEBUG) cat("\nStart permutation test for significant topRanks. \n")	
 		
@@ -218,33 +225,81 @@ classify.stsvm <- function(fold, cuts,ex.sum, x, p,a, y, cv.repeat, DEBUG=DEBUG,
 			for(j in topGenes) if(rankt[j] >= topRanks[j])p_ranks[j] = p_ranks[j]+1					
 		}	
 		cat("head(p_ranks)   ", head(p_ranks),"\n")
-		p_ranks = p_ranks/R
-		#cat("head(p_ranks)/R ", head(p_ranks),"\n")
-		#cat("range(rankt) ", range(rankt),"\n")
-		#cat("head(rankt) ", head(rankt),"\n")
-		p_ranks = p.adjust(p_ranks, method="BH")
+		p_ranks = p_ranks/R	
+		#p_ranks = p.adjust(p_ranks, method="BH")
 		names(p_ranks)=names(topRanks)
-		
+	
 		sigRank <- names(p_ranks[which(p_ranks < dk.tf)])		
-		if(length(sigRank)==0)sigRank=names(sort(topRanks))[1:10]							
-		
-		feat  = sigRank					
-		
-		train 	 <- svm.fit(x=x[trn,feat], y=y[trn], Cs=Cs, scale="scale", DEBUG=FALSE) 
-		test    <- svm.predict(fit=train, newdata=x[tst,], type="response")
-		
-		auc	<- calc.auc(test, label)				
+		if(length(sigRank)==0)sigRank=names(sort(topRanks))[1:10]									
+		feat  = sigRank	
+		train 	 <- svm.fit(x=x[,feat], y=y, Cs=Cs, scale="scale", DEBUG=FALSE) 					
+	}
+	
+	if(op.method=="sp")
+	{
+		fits  <- list()
+	  	best.boundL <- list()
+	  	featL  <- list()
+	 
+		if(DEBUG) cat("Geting Gene Ranking \n")			
+		#ranks = getGeneRanking(x=x, y=y, Gsub=Gsub, d=d)			
+	
+		ranksI=sort(ranks,decreasing=T)
+		#nodesI=sort(topNodes,decreasing=T)
 			
-		#auc3=spec
-		if(DEBUG) cat("fold:",fold,"\tlength(test):",length(test),"\tlength(label)",length(label),"\n")			
-	
-		if(DEBUG) cat("\n\n=> the best AUC	is ", auc, "\t Best features length:  ", length(feat),"\n\n")			
-		if(DEBUG) {cat('Finished fold:',fold,'\n\n')}
-		#else {setTxtProgressBar(pb, getTxtProgressBar(pb) + 1)}
-		gc()	
-		res=list(fold=fold, model=train, auc=auc, feat= feat)
+		i=1
+		nn=aa		
+		while(nn > op)
+		{ 		  				  		
+				feat.rank = names(ranksI[1:nn])	  	
+		  		feat = feat.rank
+		  		featL[[i]] = feat			 
+			  	#print(length(feat))	
+		  		fit <- svm.fit(x=x[,feat], y=y, Cs=Cs, scale="scale", DEBUG=FALSE) 	  		  
+		  		fits[[i]]=fit	  		  		
+				best.boundL[[i]] <-  fits[[i]]$error.bound  
+				nn=nn-3
+				i=i+1
+		}	
+		
+		if(DEBUG)cat("the opitimal steps: ", i-1, "\n")
+		
+		best.boundLs= unlist(best.boundL)
+		best.index = which(best.boundLs==min(best.boundLs))		
+		n=length(best.index)						
+			
+		train   =	fits[[best.index[n]]]		
+		feat = featL[[best.index[n]]]	
+	}	
+
+	if ( !is.null(EN2SY))
+	{		
+		syb = EN2SY[feat,2]
+		sGsub=Gsub[feat,feat]
+		colnames(sGsub) = syb
+		rownames(sGsub) = syb
+		train$graph = graph.adjacency(sGsub, mode="undirected")		
+	}
+	else
+	{
+		train$graph = graph.adjacency(Gsub[feat,feat], mode="undirected")	
+	}
+	train$feat=feat	
 	
+	res=list(trained=train, feat= feat)		
+	return(res)
+}
+
+
+
+predictStsvm <- function(train=train, x=x, y=y,DEBUG=DEBUG)
+{	
+	label 	<- sign(as.numeric(y) - 1.5) # because y is a factor			
 	
+	feat  = train$feat				
+	test    <- svm.predict(fit=train$trained, newdata=x, type="response")		
+	auc	<- calc.auc(test, label)				
+	res=list(auc=auc,pred=test)	
 	return(res)
 }
 
